import { r as registerInstance, c as createEvent, h, H as Host } from './index-b72aa8e0.js';
import { I as ImageRegion } from './utils-79a5728e.js';
var k2btImageRegionSelectorCss = ":host{display:block}.K2BT_LocationInBlueprintSelectorContainer{background:#eee;position:relative;background-repeat:no-repeat;background-size:100% 100%}.K2BT_LocationInBlueprintSelectorItem{position:absolute;background-repeat:no-repeat;background-size:100% 100%}.K2BT_LocationInBlueprintSelectorItemName{display:none}.K2BT_LocationInBlueprintSelectorContainer .tooltip-inner{white-space:pre-wrap}.K2BT_LocationInBlueprintSelectorItemUnavailable{cursor:initial}.K2BT_LocationInBlueprintSelectorItemUnavailable{pointer-events:none}";
var K2btImageRegionSelector = /** @class */ (function () {
    function K2btImageRegionSelector(hostRef) {
        registerInstance(this, hostRef);
        this.selectionChangedEvent = createEvent(this, "selectionChanged", 7);
        this.selectionErrorEvent = createEvent(this, "selectionError", 7);
        this.maxSelectionSize = 1;
        this.elementsWithTooltips = [];
        this.elementDivs = [];
    }
    K2btImageRegionSelector.prototype.getRegions = function () {
        var _a, _b;
        return (_b = (_a = this.imageWithRegionsDefinition) === null || _a === void 0 ? void 0 : _a.Regions) !== null && _b !== void 0 ? _b : [];
    };
    K2btImageRegionSelector.prototype.itemClicked = function (id) {
        var element = this.getRegions().filter(function (r) { return r.id == id; })[0];
        if (element.status == ImageRegion.AVAILABLE) {
            if (this.selectedIds.includes(id)) {
                // It is always possible to remove items
                this.selectedIds = this.selectedIds.filter(function (i) { return i != id; });
                this.selectionChangedEvent.emit({});
            }
            else {
                // Before selecting this item, check that the amount of items is OK
                if (this.maxSelectionSize == 1) {
                    this.selectedIds = [id];
                    this.selectionChangedEvent.emit({});
                }
                else if (this.maxSelectionSize == 0 || this.maxSelectionSize > this.selectedIds.length) {
                    this.selectedIds = this.selectedIds.concat(id);
                    this.selectionChangedEvent.emit({});
                }
                else {
                    this.errorCode = K2btImageRegionSelector.ERROR_SELECTION_FULL;
                    this.selectionErrorEvent.emit({});
                }
            }
        }
    };
    K2btImageRegionSelector.prototype.componentDidRender = function () {
        // @ts-ignore
        $(this.elementsWithTooltips).tooltip();
    };
    K2btImageRegionSelector.prototype.render = function () {
        var _this = this;
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        this.elementsWithTooltips = [];
        this.elementDivs = [];
        if (this.imageWithRegionsDefinition) {
            return (h(Host, null, h("div", { class: (_c = 'K2BT_LocationInBlueprintSelectorContainer ' + ((_b = (_a = this.imageWithRegionsDefinition) === null || _a === void 0 ? void 0 : _a.Frame) === null || _b === void 0 ? void 0 : _b.class)) !== null && _c !== void 0 ? _c : '', style: {
                    height: (_e = (_d = this.imageWithRegionsDefinition) === null || _d === void 0 ? void 0 : _d.Frame) === null || _e === void 0 ? void 0 : _e.height,
                    width: (_g = (_f = this.imageWithRegionsDefinition) === null || _f === void 0 ? void 0 : _f.Frame) === null || _g === void 0 ? void 0 : _g.width,
                    backgroundImage: ((_k = (_j = (_h = this.imageWithRegionsDefinition) === null || _h === void 0 ? void 0 : _h.Frame) === null || _j === void 0 ? void 0 : _j.backgroundImageURL) !== null && _k !== void 0 ? _k : '' != '') ? 'url(' + this.imageWithRegionsDefinition.Frame.backgroundImageURL + ')' : null,
                } }, this.getRegions().map(function (r) {
                var imageUrl = _this.getBestImageForItemStatus(r);
                var style = {
                    backgroundImage: '',
                    height: r.height,
                    width: r.width,
                    top: r.top,
                    left: r.left,
                };
                return (h("div", { class: 'K2BT_LocationInBlueprintSelectorItem' +
                        (_this.selectedIds.includes(r.id) ? ' K2BT_LocationInBlueprintSelectorItemSelected' : '') +
                        (r.status == ImageRegion.UNAVAILABLE ? ' K2BT_LocationInBlueprintSelectorItemUnavailable' : '') +
                        (r.class != '' ? ' ' + r.class : ''), ref: function (e) { return _this.elementDivs.push({ id: r.id, element: e }); }, style: style }, _this.getMapForItem(r), _this.getImage(r, imageUrl), h("span", { class: "K2BT_LocationInBlueprintSelectorItemName" }, r.name)));
            }))));
        }
        else {
            return h("div", null, "Image definition not set");
        }
    };
    K2btImageRegionSelector.prototype.getBestImageForItemStatus = function (r) {
        if (this.selectedIds.includes(r.id) && r.selectedImageURL != '')
            return r.selectedImageURL;
        if (r.status == ImageRegion.UNAVAILABLE && r.unavailableImageURL != '')
            return r.unavailableImageURL;
        return r.availableImageURL;
    };
    K2btImageRegionSelector.prototype.getImage = function (r, imageUrl) {
        var _this = this;
        return (h("img", { ref: function (e) {
                if (r.selectableRegionCoordinates == '')
                    _this.elementsWithTooltips.push(e);
            }, title: r.selectableRegionCoordinates == '' ? this.getTooltipLines(r) : '', src: imageUrl, style: { height: '100%', width: '100%', cursor: r.selectableRegionCoordinates == '' ? 'pointer' : null }, usemap: r.selectableRegionCoordinates != '' ? '#K2BT_Image' + r.id : null, onClick: function () {
                if (r.selectableRegionCoordinates == '')
                    _this.itemClicked(r.id);
            } }));
    };
    K2btImageRegionSelector.prototype.getTooltipLines = function (r) {
        if (r.tooltipLines == null || r.tooltipLines.length == 0)
            return null;
        else
            return r.tooltipLines.join('\n');
    };
    K2btImageRegionSelector.prototype.getMapForItem = function (r) {
        var _this = this;
        if (r.selectableRegionCoordinates != '') {
            return (h("map", { name: 'K2BT_Image' + r.id }, h("area", { shape: "poly", coords: r.selectableRegionCoordinates, onClick: function () { return _this.itemClicked(r.id); }, href: "#", title: this.getTooltipLines(r), ref: function (c) { return _this.elementsWithTooltips.push(c); } }), h("area", { shape: "poly", coords: '0,0,' + r.width + ',0,' + r.width + ',' + r.height + ',0,' + r.height, onClick: function (e) { return _this.clickBelow(e, r.id); }, onMouseOver: function (e) { return _this.bringElementBelowToForeground(e, r.id); } })));
        }
    };
    K2btImageRegionSelector.prototype.bringElementBelowToForeground = function (e, regionId) {
        var evt = e || window.event;
        var regionDiv = this.elementDivs.filter(function (d) { return d.id == regionId; })[0].element;
        regionDiv.style.pointerEvents = 'none';
        // get element at point of click
        var starter = document.elementFromPoint(evt.clientX, evt.clientY);
        var elements = this.elementDivs.filter(function (e) { return e.element.contains(starter); });
        if (elements.length > 0) {
            console.log('hovering: ' + elements[0].id);
            this.elementDivs.forEach(function (el) { return (el.element.style.zIndex = 'initial'); });
            elements.forEach(function (el) { return (el.element.style.zIndex = '1'); });
        }
        // bring back the cursor
        regionDiv.style.pointerEvents = 'auto';
    };
    K2btImageRegionSelector.prototype.clickBelow = function (e, regionId) {
        var evt = e || window.event;
        var regionDiv = this.elementDivs.filter(function (d) { return d.id == regionId; })[0].element;
        regionDiv.style.pointerEvents = 'none';
        // get element at point of click
        var starter = document.elementFromPoint(evt.clientX, evt.clientY);
        var elements = this.elementDivs.filter(function (e) { return e.element.contains(starter); });
        if (elements.length > 0)
            this.itemClicked(elements[0].id);
        // bring back the cursor
        regionDiv.style.pointerEvents = 'auto';
    };
    return K2btImageRegionSelector;
}());
K2btImageRegionSelector.ERROR_SELECTION_FULL = 'SELECTION_FULL';
K2btImageRegionSelector.style = k2btImageRegionSelectorCss;
export { K2btImageRegionSelector as k2bt_image_region_selector };
